@@ -28,9 +28,7 @@ fn twos_complement(u: U256) -> U256 {
 fn handle_overflow<T>((result, overflow): (T, bool)) -> T {
     #[cfg(debug_assertions)]
     {
-        if overflow {
-            panic!("overflow");
-        }
+        assert!(!overflow, "overflow");
     }
 
     let _ = overflow;
@@ -137,6 +135,7 @@ impl I256 {
     /// Coerces an unsigned integer into a signed one. If the unsigned integer
     /// is greater than the greater than or equal to `1 << 255`, then the result
     /// will overflow into a negative value.
+    #[must_use]
     pub fn from_raw(raw: U256) -> Self {
         I256(raw)
     }
@@ -144,46 +143,55 @@ impl I256 {
     /// Returns the signed integer as a unsigned integer. If the value of `self`
     /// negative, then the two's complement of its absolute value will be
     /// returned.
+    #[must_use]
     pub fn into_raw(self) -> U256 {
         self.0
     }
 
     /// Conversion to i32
+    #[must_use]
     pub fn low_i32(&self) -> i32 {
         self.0.low_u32() as _
     }
 
     /// Conversion to u32
+    #[must_use]
     pub fn low_u32(&self) -> u32 {
         self.0.low_u32()
     }
 
     /// Conversion to i64
+    #[must_use]
     pub fn low_i64(&self) -> i64 {
         self.0.low_u64() as _
     }
 
     /// Conversion to u64
+    #[must_use]
     pub fn low_u64(&self) -> u64 {
         self.0.low_u64() as _
     }
 
     /// Conversion to i128
+    #[must_use]
     pub fn low_i128(&self) -> i128 {
         self.0.low_u128() as _
     }
 
     /// Conversion to u128
+    #[must_use]
     pub fn low_u128(&self) -> u128 {
         self.0.low_u128() as _
     }
 
     /// Conversion to i128
+    #[must_use]
     pub fn low_isize(&self) -> isize {
         self.0.low_u64() as _
     }
 
     /// Conversion to usize
+    #[must_use]
     pub fn low_usize(&self) -> usize {
         self.0.low_u64() as _
     }
@@ -193,6 +201,7 @@ impl I256 {
     /// # Panics
     ///
     /// Panics if the number is outside the range [`i32::MIN`, `i32::MAX`].
+    #[must_use]
     pub fn as_i32(&self) -> i32 {
         (*self).try_into().unwrap()
     }
@@ -202,6 +211,7 @@ impl I256 {
     /// # Panics
     ///
     /// Panics if the number is outside the range [`0`, `u32::MAX`].
+    #[must_use]
     pub fn as_u32(&self) -> u32 {
         (*self).try_into().unwrap()
     }
@@ -211,6 +221,7 @@ impl I256 {
     /// # Panics
     ///
     /// Panics if the number is outside the range [`i64::MIN`, `i64::MAX`].
+    #[must_use]
     pub fn as_i64(&self) -> i64 {
         (*self).try_into().unwrap()
     }
@@ -220,6 +231,7 @@ impl I256 {
     /// # Panics
     ///
     /// Panics if the number is outside the range [`0`, `u64::MAX`].
+    #[must_use]
     pub fn as_u64(&self) -> u64 {
         (*self).try_into().unwrap()
     }
@@ -229,6 +241,7 @@ impl I256 {
     /// # Panics
     ///
     /// Panics if the number is outside the range [`i128::MIN`, `i128::MAX`].
+    #[must_use]
     pub fn as_i128(&self) -> i128 {
         (*self).try_into().unwrap()
     }
@@ -238,6 +251,7 @@ impl I256 {
     /// # Panics
     ///
     /// Panics if the number is outside the range [`0`, `u128::MAX`].
+    #[must_use]
     pub fn as_u128(&self) -> u128 {
         (*self).try_into().unwrap()
     }
@@ -247,6 +261,7 @@ impl I256 {
     /// # Panics
     ///
     /// Panics if the number is outside the range [`isize::MIN`, `isize::MAX`].
+    #[must_use]
     pub fn as_isize(&self) -> usize {
         (*self).try_into().unwrap()
     }
@@ -256,6 +271,7 @@ impl I256 {
     /// # Panics
     ///
     /// Panics if the number is outside the range [`0`, `usize::MAX`].
+    #[must_use]
     pub fn as_usize(&self) -> usize {
         (*self).try_into().unwrap()
     }
@@ -300,6 +316,7 @@ impl I256 {
     }
 
     /// Returns the sign of the number.
+    #[must_use]
     pub fn signum(self) -> Self {
         self.signum64().into()
     }
@@ -320,18 +337,21 @@ impl I256 {
 
     /// Returns `true` if `self` is positive and `false` if the number is zero
     /// or negative.
+    #[must_use]
     pub fn is_positive(self) -> bool {
         self.signum64().is_positive()
     }
 
     /// Returns `true` if `self` is negative and `false` if the number is zero
     /// or negative.
+    #[must_use]
     pub fn is_negative(self) -> bool {
         self.signum64().is_negative()
     }
 
     /// Returns `true` if `self` is negative and `false` if the number is zero
     /// or positive.
+    #[must_use]
     pub fn is_zero(self) -> bool {
         self.0.is_zero()
     }
@@ -341,6 +361,7 @@ impl I256 {
     /// # Panics
     ///
     /// In debug mode, will panic if it overflows.
+    #[must_use]
     pub fn abs(self) -> Self {
         handle_overflow(self.overflowing_abs())
     }
@@ -352,6 +373,7 @@ impl I256 {
     /// (e.g., `I256::MIN` for values of type `I256`), then the minimum value
     /// will be returned again and true will be returned for an overflow
     /// happening.
+    #[must_use]
     pub fn overflowing_abs(self) -> (Self, bool) {
         if self == I256::MIN {
             (self, true)
@@ -362,6 +384,7 @@ impl I256 {
 
     /// Checked absolute value. Computes `self.abs()`, returning `None` if
     /// `self == MIN`.
+    #[must_use]
     pub fn checked_abs(self) -> Option<Self> {
         let (result, overflow) = self.overflowing_abs();
         if overflow {
@@ -373,12 +396,14 @@ impl I256 {
 
     /// Saturating absolute value. Computes `self.abs()`, returning `MAX` if
     /// `self == MIN` instead of overflowing.
+    #[must_use]
     pub fn saturating_abs(self) -> Self {
         self.checked_abs().unwrap_or(I256::MAX)
     }
 
     /// Wrapping absolute value. Computes `self.abs()`, wrapping around at the
     /// boundary of the type.
+    #[must_use]
     pub fn wrapping_abs(self) -> Self {
         let (result, _) = self.overflowing_abs();
         result
@@ -396,6 +421,7 @@ impl I256 {
     /// indicating whether an overflow happened. If `self` is the minimum value,
     /// then the minimum value will be returned again and `true` will be
     /// returned for an overflow happening.
+    #[must_use]
     pub fn overflowing_neg(self) -> (Self, bool) {
         if self == I256::MIN {
             (self, true)
@@ -406,6 +432,7 @@ impl I256 {
 
     /// Checked negation. Computes `self.neg()`, returning `None` if
     /// `self == MIN`.
+    #[must_use]
     pub fn checked_neg(self) -> Option<Self> {
         let (result, overflow) = self.overflowing_neg();
         if overflow {
@@ -417,18 +444,21 @@ impl I256 {
 
     /// Saturating negation. Computes `self.neg()`, returning `MAX` if
     /// `self == MIN` instead of overflowing.
+    #[must_use]
     pub fn saturating_neg(self) -> Self {
         self.checked_neg().unwrap_or(I256::MAX)
     }
 
     /// Wrapping negation. Computes `self.neg()`, returning `MIN` if
     /// `self == MIN` instead of overflowing.
+    #[must_use]
     pub fn wrapping_neg(self) -> Self {
         let (result, _) = self.overflowing_neg();
         result
     }
 
     /// Return the least number of bits needed to represent the number.
+    #[must_use]
     pub fn bits(&self) -> u32 {
         let unsigned = self.abs_unsigned();
         let unsigned_bits = unsigned.bits();
@@ -463,28 +493,33 @@ impl I256 {
     /// # Panics
     ///
     /// Panics if index exceeds the bit width of the number.
+    #[must_use]
     pub fn bit(&self, index: usize) -> bool {
         self.0.bit(index)
     }
 
     /// Returns the number of ones in the binary representation of `self`.
+    #[must_use]
     pub fn count_ones(&self) -> u32 {
         (self.0).0.iter().map(|word| word.count_ones()).sum()
     }
 
     /// Returns the number of zeros in the binary representation of `self`.
+    #[must_use]
     pub fn count_zeros(&self) -> u32 {
         (self.0).0.iter().map(|word| word.count_zeros()).sum()
     }
 
     /// Returns the number of leading zeros in the binary representation of
     /// `self`.
+    #[must_use]
     pub fn leading_zeros(&self) -> u32 {
         self.0.leading_zeros()
     }
 
     /// Returns the number of leading zeros in the binary representation of
     /// `self`.
+    #[must_use]
     pub fn trailing_zeros(&self) -> u32 {
         self.0.trailing_zeros()
     }
@@ -494,6 +529,7 @@ impl I256 {
     /// # Panics
     ///
     /// Panics if index exceeds the byte width of the number.
+    #[must_use]
     pub fn byte(&self, index: usize) -> u8 {
         self.0.byte(index)
     }
@@ -515,6 +551,7 @@ impl I256 {
     /// Returns a tuple of the addition along with a boolean indicating whether
     /// an arithmetic overflow would occur. If an overflow would have occurred
     /// then the wrapped value is returned.
+    #[must_use]
     pub fn overflowing_add(self, rhs: Self) -> (Self, bool) {
         let (unsigned, _) = self.0.overflowing_add(rhs.0);
         let result = I256(unsigned);
@@ -531,6 +568,7 @@ impl I256 {
     }
 
     /// Checked addition. Returns None if overflow occurred.
+    #[must_use]
     pub fn checked_add(self, other: Self) -> Option<Self> {
         let (result, overflow) = self.overflowing_add(other);
         if overflow {
@@ -541,6 +579,7 @@ impl I256 {
     }
 
     /// Addition which saturates at the maximum value (Self::max_value()).
+    #[must_use]
     pub fn saturating_add(self, other: Self) -> Self {
         let (result, overflow) = self.overflowing_add(other);
         if overflow {
@@ -554,6 +593,7 @@ impl I256 {
     }
 
     /// Wrapping addition.
+    #[must_use]
     pub fn wrapping_add(self, other: Self) -> Self {
         let (result, _) = self.overflowing_add(other);
         result
@@ -564,6 +604,7 @@ impl I256 {
     /// Returns a tuple of the subtraction along with a boolean indicating
     /// whether an arithmetic overflow would occur. If an overflow would have
     /// occurred then the wrapped value is returned.
+    #[must_use]
     pub fn overflowing_sub(self, rhs: Self) -> (Self, bool) {
         // NOTE: We can't just compute the `self + (-rhs)` because `-rhs` does
         //   not always exist, specifically this would be a problem in case
@@ -584,6 +625,7 @@ impl I256 {
     }
 
     /// Checked subtraction. Returns None if overflow occurred.
+    #[must_use]
     pub fn checked_sub(self, other: Self) -> Option<Self> {
         let (result, overflow) = self.overflowing_sub(other);
         if overflow {
@@ -594,6 +636,7 @@ impl I256 {
     }
 
     /// Subtraction which saturates at zero.
+    #[must_use]
     pub fn saturating_sub(self, other: Self) -> Self {
         let (result, overflow) = self.overflowing_sub(other);
         if overflow {
@@ -607,6 +650,7 @@ impl I256 {
     }
 
     /// Wrapping subtraction.
+    #[must_use]
     pub fn wrapping_sub(self, other: Self) -> Self {
         let (result, _) = self.overflowing_sub(other);
         result
@@ -617,6 +661,7 @@ impl I256 {
     /// Returns a tuple of the multiplication along with a boolean indicating
     /// whether an arithmetic overflow would occur. If an overflow would have
     /// occurred then the wrapped value is returned.
+    #[must_use]
     pub fn overflowing_mul(self, rhs: Self) -> (Self, bool) {
         let sign = Sign::from_signum64(self.signum64() * rhs.signum64());
         let (unsigned, overflow_mul) = self.abs_unsigned().overflowing_mul(rhs.abs_unsigned());
@@ -626,6 +671,7 @@ impl I256 {
     }
 
     /// Checked multiplication. Returns None if overflow occurred.
+    #[must_use]
     pub fn checked_mul(self, other: Self) -> Option<Self> {
         let (result, overflow) = self.overflowing_mul(other);
         if overflow {
@@ -635,7 +681,8 @@ impl I256 {
         }
     }
 
-    /// Multiplication which saturates at the maximum value..
+    /// Multiplication which saturates at the maximum value.
+    #[must_use]
     pub fn saturating_mul(self, rhs: Self) -> Self {
         self.checked_mul(rhs).unwrap_or_else(|| {
             match Sign::from_signum64(self.signum64() * rhs.signum64()) {
@@ -646,6 +693,7 @@ impl I256 {
     }
 
     /// Wrapping multiplication.
+    #[must_use]
     pub fn wrapping_mul(self, rhs: Self) -> Self {
         let (result, _) = self.overflowing_mul(rhs);
         result
@@ -656,6 +704,7 @@ impl I256 {
     /// Returns a tuple of the division along with a boolean indicating
     /// whether an arithmetic overflow would occur. If an overflow would have
     /// occurred then the wrapped value is returned.
+    #[must_use]
     pub fn overflowing_div(self, rhs: Self) -> (Self, bool) {
         // Panic early when with division by zero while evaluating sign.
         let sign = Sign::from_signum64(self.signum64() / rhs.signum64());
@@ -667,6 +716,7 @@ impl I256 {
     }
 
     /// Checked division. Returns None if overflow occurred or if rhs == 0.
+    #[must_use]
     pub fn checked_div(self, rhs: Self) -> Option<Self> {
         if rhs == I256::zero() || (self == Self::min_value() && rhs == -I256::one()) {
             None
@@ -676,12 +726,14 @@ impl I256 {
     }
 
     /// Division which saturates at the maximum value.
+    #[must_use]
     pub fn saturating_div(self, rhs: Self) -> Self {
         // There is only one overflow (I256::MIN / -1 = I256::MAX)
         self.checked_div(rhs).unwrap_or(I256::MAX)
     }
 
     /// Wrapping division.
+    #[must_use]
     pub fn wrapping_div(self, rhs: Self) -> Self {
         self.overflowing_div(rhs).0
     }
@@ -691,6 +743,7 @@ impl I256 {
     /// Returns a tuple of the remainder along with a boolean indicating
     /// whether an arithmetic overflow would occur. If an overflow would have
     /// occurred then the wrapped value is returned.
+    #[must_use]
     pub fn overflowing_rem(self, rhs: Self) -> (Self, bool) {
         if self == Self::MIN && rhs == Self::from(-1) {
             (Self::zero(), true)
@@ -701,6 +754,7 @@ impl I256 {
     }
 
     /// Checked remainder. Returns None if overflow occurred or rhs == 0
+    #[must_use]
     pub fn checked_rem(self, rhs: Self) -> Option<Self> {
         if rhs == I256::zero() || (self == Self::min_value() && rhs == -I256::one()) {
             None
@@ -711,6 +765,7 @@ impl I256 {
 
     /// Wrapping remainder. Returns the result of the operation %
     /// regardless of whether or not the division overflowed.
+    #[must_use]
     pub fn wrapping_rem(self, rhs: Self) -> Self {
         self.overflowing_rem(rhs).0
     }
@@ -722,6 +777,7 @@ impl I256 {
     /// In other words, the result is `self / rhs` rounded to the integer `n` such that `self >= n * rhs`:
     /// * If `self > 0`, this is equal to round towards zero (the default in Rust);
     /// * If `self < 0`, this is equal to round towards +/- infinity.
+    #[must_use]
     pub fn div_euclid(self, rhs: Self) -> Self {
         let q = self / rhs;
         if (self % rhs).is_negative() {
@@ -737,6 +793,7 @@ impl I256 {
     /// Calculates the least non-negative remainder of self (mod rhs).
     /// This is done as if by the _Euclidean division algorithm_
     /// given `r = self.rem_euclid(rhs)`, `self = rhs * self.div_euclid(rhs) + r, and 0 <= r < abs(rhs)`.
+    #[must_use]
     pub fn rem_euclid(self, rhs: Self) -> Self {
         let r = self % rhs;
         if r < Self::zero() {
@@ -753,6 +810,7 @@ impl I256 {
     /// Calculates the quotient of Euclidean division `self.div_euclid(rhs)`.
     /// Returns a tuple of the divisor along with a boolean indicating whether an arithmetic
     /// overflow would occur. If an overflow would occur then `self` is returned.
+    #[must_use]
     pub fn overflowing_div_euclid(self, rhs: Self) -> (Self, bool) {
         if self == Self::min_value() && rhs == -I256::one() {
             (self, true)
@@ -763,6 +821,7 @@ impl I256 {
 
     /// Checked Euclidean division. Computes `self.div_euclid(rhs)`,
     /// returning None if `rhs == 0` or the division results in overflow.
+    #[must_use]
     pub fn checked_div_euclid(self, rhs: Self) -> Option<Self> {
         if rhs == I256::zero() || (self == Self::min_value() && rhs == -I256::one()) {
             None
@@ -777,6 +836,7 @@ impl I256 {
     /// (where `MIN` is the negative minimal value for the type).
     /// This is equivalent to `-MIN`, a positive value that is too large to represent in the type.
     /// In this case, this method returns `MIN` itself.
+    #[must_use]
     pub fn wrapping_div_euclid(self, rhs: Self) -> Self {
         self.overflowing_div_euclid(rhs).0
     }
@@ -785,6 +845,7 @@ impl I256 {
     /// Returns a tuple of the remainder after dividing along with a boolean indicating whether
     /// an arithmetic overflow would occur. If an overflow would occur then `0` is returned.
     /// Panics if `rhs == 0`
+    #[must_use]
     pub fn overflowing_rem_euclid(self, rhs: Self) -> (Self, bool) {
         if self == Self::min_value() && rhs == -Self::one() {
             (Self::zero(), true)
@@ -799,12 +860,14 @@ impl I256 {
     /// (where `MIN` is the negative minimal value for the type).
     /// In this case, this method returns `0`.
     /// Panics when `rhs == 0`
+    #[must_use]
     pub fn wrapping_rem_euclid(self, rhs: Self) -> Self {
         self.overflowing_rem_euclid(rhs).0
     }
 
     /// Checked Euclidean remainder. Computes `self.rem_euclid(rhs)`,
     /// returning `None` if `rhs == 0` or the division results in overflow.
+    #[must_use]
     pub fn checked_rem_euclid(self, rhs: Self) -> Option<Self> {
         if rhs == I256::zero() || (self == Self::min_value() && rhs == -I256::one()) {
             None
@@ -834,6 +897,7 @@ impl I256 {
     /// # Panics
     ///
     /// Panics if the result overflows the type.
+    #[must_use]
     pub fn exp10(n: usize) -> Self {
         U256::exp10(n).try_into().expect("overflow")
     }
@@ -843,6 +907,7 @@ impl I256 {
     /// # Panics
     ///
     /// Panics if the result overflows the type in debug mode.
+    #[must_use]
     pub fn pow(self, exp: u32) -> Self {
         handle_overflow(self.overflowing_pow(exp))
     }
@@ -851,6 +916,7 @@ impl I256 {
     ///
     /// Returns a tuple of the exponentiation along with a bool indicating
     /// whether an overflow happened.
+    #[must_use]
     pub fn overflowing_pow(self, exp: u32) -> (Self, bool) {
         let sign = self.pow_sign(exp);
         let (unsigned, overflow_pow) = self.abs_unsigned().overflowing_pow(exp.into());
@@ -860,6 +926,7 @@ impl I256 {
     }
 
     /// Raises self to the power of `exp`. Returns None if overflow occurred.
+    #[must_use]
     pub fn checked_pow(self, exp: u32) -> Option<Self> {
         let (result, overflow) = self.overflowing_pow(exp);
         if overflow {
@@ -871,6 +938,7 @@ impl I256 {
 
     /// Raises self to the power of `exp`, saturating at the numeric bounds
     /// instead of overflowing.
+    #[must_use]
     pub fn saturating_pow(self, exp: u32) -> Self {
         let (result, overflow) = self.overflowing_pow(exp);
         if overflow {
@@ -885,6 +953,7 @@ impl I256 {
 
     /// Wrapping powolute value. Computes `self.pow()`, wrapping around at the
     /// boundary of the type.
+    #[must_use]
     pub fn wrapping_pow(self, exp: u32) -> Self {
         let (result, _) = self.overflowing_pow(exp);
         result
@@ -1637,7 +1706,7 @@ mod tests {
     #[test]
     #[cfg_attr(debug_assertions, should_panic)]
     fn div_euclid_overflow() {
-        I256::MIN.div_euclid(-I256::one());
+        let _ = I256::MIN.div_euclid(-I256::one());
     }
 
     #[test]
