@@ -2,8 +2,7 @@
 
 use crate::errors::InvalidPrivateKey;
 use ethcontract_common::hash;
-use secp256k1::key::ONE_KEY;
-use secp256k1::{Message, PublicKey, Secp256k1, SecretKey};
+use secp256k1::{Message, PublicKey, Secp256k1, SecretKey, ONE_KEY};
 use std::fmt::{self, Debug, Formatter};
 use std::ops::Deref;
 use std::str::FromStr;
@@ -81,19 +80,28 @@ impl Debug for PrivateKey {
     }
 }
 
+// Taken from rust-web3's signing.rs.
 impl Key for &'_ PrivateKey {
     fn sign(&self, message: &[u8], chain_id: Option<u64>) -> Result<Signature, SigningError> {
+        let signature = self.sign_message(message)?;
+
+        Ok(Signature {
+            v: if let Some(chain_id) = chain_id {
+                signature.v + 35 + chain_id * 2
+            } else {
+                signature.v + 27
+            },
+            ..signature
+        })
+    }
+
+    fn sign_message(&self, message: &[u8]) -> Result<Signature, SigningError> {
         let message = Message::from_slice(message).map_err(|_| SigningError::InvalidMessage)?;
         let (recovery_id, signature) = Secp256k1::signing_only()
-            .sign_recoverable(&message, self)
+            .sign_ecdsa_recoverable(&message, self)
             .serialize_compact();
 
-        let standard_v = recovery_id.to_i32() as u64;
-        let v = if let Some(chain_id) = chain_id {
-            standard_v + 35 + chain_id * 2
-        } else {
-            standard_v + 27
-        };
+        let v = recovery_id.to_i32() as u64;
         let r = H256::from_slice(&signature[..32]);
         let s = H256::from_slice(&signature[32..]);
 
