@@ -17,6 +17,7 @@ use web3::Transport;
 
 /// A struct with the confirmation parameters.
 #[derive(Clone, Debug)]
+#[must_use = "confirm parameters do nothing unless waited for"]
 pub struct ConfirmParams {
     /// The number of blocks to confirm the transaction with. This is the number
     /// of blocks mined on top of the block where the transaction was mined.
@@ -298,6 +299,7 @@ mod tests {
             "gasUsed": "0x1337",
             "logsBloom": H2048::zero(),
             "logs": [],
+            "effectiveGasPrice": "0x0",
         })
     }
 
